import ColumnObject from './ColumnObject/ColumnObject';

export default class ColumnArray {
  constructor({ columns = [] }) {
    this.columnList = {};
    this.numberofColumns = 0;
    for (let i = 0, length = columns.length, tempColumn; i < length; i += 1) {
      // If column already have an index number then keep that one
      // Else use the numberofColumns number as the column index.
      columns[i].index = columns[i].index || this.numberofColumns;
      tempColumn = new ColumnObject(columns[i]);
      this.columnList[tempColumn.name] = tempColumn;
      this.numberofColumns += 1
    }
  }
  
  getColumn(columnName) {
    return this.columnList[columnName];
  }

  getComputedColumns() {
    const results = [];
    const columnNames = Object.keys(this.columnList);
    for (let i = 0, column; i < columnNames.length; i += 1) {
      column = this.columnList[columnNames[i]];
      if (column.type === 'Computed') {
        results.push(column);
      }
    }
    return results;
  }

  getColumnByIndex(index) {
    const columnNames = Object.keys(this.columnList);
    for (let i = 0, length = columnNames.length, column; i < length; i += 1) {
      column = this.columnList[columnNames[i]];
      if (column.index === index) {
        return column;
      }
    }
    return false;
  }

  getColumnIndex(columnName) {
    return this.columnList[columnName].getColumnIndex();
  }

  getMultipleColumnIndex(columnNameList) {
    const results = [];
    for (let i = 0, length = columnNameList.length; i < length; i += 1) {
      results.push(this.getColumnIndex(columnNameList[i]));
    }
    return results;
  }

  addColumn(name, type, index) {
    this.columnList[name] = new ColumnObject({ name, type, index });
    this.numberofColumns = this.getNumberofColumns();
  }

  pushColumn(name, type, expression = '') {
    this.columnList[name] = new ColumnObject({ name, type, expression, index: this.getNumberofColumns() });
    this.numberofColumns = this.getNumberofColumns();
  }

  removeColumn(name) {
    delete this.columnList[name];
  }

  removeMultipleColumns(columnNameList) {
    while(columnNameList.length) {
      let columnName = columnNameList.shift();
      delete this.columnList[columnName];
    }
  }

  editColumnName(oldColumnName, newColumnName) {
    const column = this.columnList[oldColumnName];
    column.name = newColumnName;
    delete this.columnList[oldColumnName];
    this.columnList[newColumnName] = column;
  }

  getNumberofColumns() {
    return Object.keys(this.columnList).length;
  }
}
